function [K,T] = OFcontrol(A,B,C,S,Kf,Qord)
% {A,B,C} denotes the state space matrices of open loop plant
% S: the sparse pattern
% Kf: a centralized strongly stabilizing controller C
% Qord: the memory length of FIR-based decentralized controller
% K: decentralized controller C_s
% T: plant [T_{uw}(C)-C_s T_{yw}(C)]

% form the closed-loop with centralized controller (step 2)

G = ss(A,[B,B],[C;C],0,-1);

zo = size(C,1);
wi = size(B,2);

yo = size(C,1);
uo = size(B,2);
ui = uo;
yi = yo;
connections = [wi+1:wi+ui+yi;zo+yo+1:zo+yo+uo,zo+1:zo+yo]';
GK = connect(append(G,Kf),connections,1:wi,zo+1:zo+yo+uo);
GK = minreal(GK);
Am = GK.a;
Bm = GK.b;
Cm = GK.c;
Dm = GK.d;
Cu = Cm(yo+1:end,:);
Cm = Cm(1:yo,:);
Dm = Dm(1:yo,:);


% pass T_{uw} and T{yw} through the next function solving step (4)
[K,T] = sparseOF(Am,Bm,Cm,Dm,Cu,S,Qord);

G = ss(A,B,C,0,-1);
fbk = feedback(G,K,+1);
if isstable(fbk)
    disp('Closed-loop is stable');
else
    disp('Closed-loop is not stable');
end
disp('OF approach finished...');