clear; close all;
% load the problem model
load('example1_discrete');

nx = size(A,1);
nu = size(B,2);
ny = size(C,1);

Bw = B;
Cz = C;

D11 = zeros(ny,nu); D12 = zeros(ny,nu);
D21 = zeros(ny,nu); D22 = zeros(ny,nu);

% -------------------------------------------------------------------------
% for discrete-time model

Gz = ss(A,[Bw,B],[Cz;C],[D11,D12;D21,D22],-1);


% -------------------------------------------------------------------------
% for continuous-time model, use the Euler approximation with a given
% sampling time

% Gz = ss(A,[Bw,B],[Cz;C],[D11,D12;D21,D22],-1);
% sample_time = 0.1;
% A = eye(size(A,1))+sample_time*A;
% Bw = sample_time*Bw;
% B = sample_time*B;

% -------------------------------------------------------------------------
% specify the sparse pattern
S = ones(nu,ny);
S(1,ny) = 0;
S(nu,1) = 0;

% -------------------------------------------------------------------------
% load the stable centralized controller we used in our experiment
load('exp1_Kf');

% -------------------------------------------------------------------------
% run our algorithm for the decentralized controller synthesis with a given
% parameter specifying the memory length of FIR

% change this parameter from 1 to 13, you should recover our result
order = 5;
KOF = OFcontrol(A,B,C,S,Kf,order);

% -------------------------------------------------------------------------
% transfer the discrete-time controller to continuous version for
% continuous-time model

% KOF = EASd2c(KOF,sample_time);

% -------------------------------------------------------------------------
% form the closed-loop system, check the stability
fbkOF = lft(Gz,KOF,nu,ny);
if isstable(fbkOF)
    fprintf('OF controller stabilizes the plant with closed-loop Hinf norm: %f\n',hinfnorm(fbkOF));
else
    disp('OF controller does not stabilize the plant.');
end